<?php
namespace Ak\ServiceManager\AbstractFactory;

use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Exception\InvalidServiceException;
use Laminas\ServiceManager\Factory\AbstractFactoryInterface;
use Laminas\ServiceManager\PluginManagerInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;

/**
 * Kreira instance plugin managera koji su konfigurirani u glavnoj konfiguraciji.
 *
 * @package Ak\ServiceManager\AbstractFactory
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/service-manager
 */
final class PluginManagerAbstractFactory implements AbstractFactoryInterface
{
    /**
     * Da li možemo napraviti instancu traženog plugin managera?
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     *
     * @return bool
     */
    public function canCreate(ContainerInterface $container, $requestedName): bool
    {
        $interfaces = class_implements($requestedName);

        return isset($interfaces[PluginManagerInterface::class]) &&
            defined($requestedName . '::CONFIG_KEY');
    }

    /**
     * Create an object
     *
     * @param ContainerInterface $container
     * @param string $requestedName
     * @param null|array $options
     *
     * @return PluginManagerInterface
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null): PluginManagerInterface
    {
        if (!$this->canCreate($container, $requestedName)) {
            throw new InvalidServiceException(sprintf(
                "Ne možemo kreirati objekte klase '%s' kao plugin managere.",
                $requestedName
            ));
        }

        $config = $container->get('config');
        $config_key = constant($requestedName . '::CONFIG_KEY');

        return new $requestedName($container, $config[$config_key] ?? []);
    }
}
